﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Diagnostics;
using System.Threading;
using OvDotNet;

namespace OvStartupProj
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        #region MainForm Events

        /// <summary>
        /// Handles the Load event of the MainForm.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            ovApi.HotLinkToOv(Environment.GetCommandLineArgs());
        }
        /// <summary>
        /// Size changed event handler, check if we just got minimized, if so hide our selfs from the taskbar!
        /// </summary>
        /// <param name="sender">this form object</param>
        /// <param name="e">default empty arguments</param>
        void MainForm_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.ShowInTaskbar = false;
            }
        }

        /// <summary>
        /// Check if the user is terminating this background form, when there is still and active link to a OV instance!
        /// </summary>
        /// <param name="sender">this form object</param>
        /// <param name="e">closing event arguments, with option to cancel.</param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = CheckIfOkToCloseAtUsersRequest();
        }
        /// <summary>
        /// Our automation form is closing, check if the link to OV
        /// has been terminated. If not then we will prompt the user
        /// if they really wish to terminate this automation.
        /// </summary>
        /// <returns>false if ok to continue shutting down, true if to abort</returns>
        private bool CheckIfOkToCloseAtUsersRequest()
        {
            bool okToShutdown = false;
            OvSessionState state = ovApi.CurrentSessionState;
            bool queryShutdown = false;
            switch (state)
            {
                //case OvSessionState.Online:
                //    break;
                //case OvSessionState.Connecting:
                //    break;
                //case OvSessionState.ConnectFailed:
                //    break;
                //case OvSessionState.Disconnected:
                //    break;
                case OvSessionState.Terminated:
                    break;
                case OvSessionState.FailedLinkage:
                    break;
                case OvSessionState.MacrosNotAllowed:
                    break;
                default:
                    queryShutdown = true;
                    break;
            }
            if (queryShutdown)
            {
                string caption = "Unanticipated Shutdown Warning!";
                string msg = "Received an unanticipated shutdown request.\r\n";
                msg += "Are you sure you wish to shutdown this automation agent?";
                DialogResult result = MessageBox.Show(msg, caption,
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Warning,
                    MessageBoxDefaultButton.Button1,
                    MessageBoxOptions.DefaultDesktopOnly);

                if (result == DialogResult.No)
                {
                    okToShutdown = true;
                }
            }
            return okToShutdown;
        }
        #endregion

        #region OutsideView Component Events

        /// <summary>
        /// Keyboard state changed event handler, look for a user forced unlock
        /// </summary>
        /// <param name="sender">ov interface component</param>
        /// <param name="e">Current keyboard state arguments</param>
        private void KeyboardStateChanged(object sender, OvKeyboardStateArgs e)
        {
            //
            //  check if the user was in a keyboard locked situation,
            //  and demanded to get out!!!
            //
            if (e.State == OvKeyboardState.UserForcedUnlock)
            {
                bgw_keyBoard.CancelAsync();
                foreach (Form form in this.OwnedForms)
                {
                    form.Close();
                }
                ovApi.ReleaseControl();
            }
        }
        /// <summary>
        /// Session state changed event handler, check if this background form should be closed.
        /// </summary>
        /// <param name="sender">ov interface component</param>
        /// <param name="e">Current session state arguments</param>
        private void SessionStateChanged(object sender, OvDotNet.OvSessionStateArgs e)
        {
            OvSessionState state = ovApi.CurrentSessionState;
            Debug.WriteLine("OVSessionState: " + state.ToString());
            bool shutdown = false;
            switch (state)
            {
                case OvSessionState.Online:
                    break;
                case OvSessionState.Connecting:
                    break;
                case OvSessionState.ConnectFailed:
                    break;
                case OvSessionState.Disconnected:
                    break;
                //case OvSessionState.Terminated:
                //    break;
                //case OvSessionState.FailedLinkage:
                //    break;
                //case OvSessionState.MacrosNotAllowed:
                //    break;
                default:
                    shutdown = true;
                    break;
            }
            if (shutdown)
            {
                this.DialogResult = DialogResult.Cancel;
                this.Close();
            }
        }

        #endregion

        #region Function Key Detection functions and event handler

        /// <summary>
        /// <para>
        /// OV API event handler, listens to function key events sent by OV to determine
        /// if user has pressed a key that we perform automations or application
        /// extensions on.
        /// </para>
        /// <para>
        /// We have a choice of consuming the key so that the terminal emulation never see it and
        /// doing our own set of actions, or doing simple actions before the emulation acts on the
        /// users key press.
        /// </para>
        /// </summary>
        /// <param name="sender">.net api component</param>
        /// <param name="e">function keystroke user pressed</param>
        void FunctionKeyPressed(object sender, KeyEventArgs e)
        {
            DebugReportFunctionKey(e, false);               //debug output the keystroke we received to act on
            bool actionKey = false;                         //assume we are not handling this keystroke
            //
            //  Check if online, if so detect if there are keys that we should take action on
            //
            if (ovApi.CurrentSessionState == OvSessionState.Online)
            {
                //
                //  Handle the normal keyboard mappings with no modifiers
                //
                if (!e.Control && !e.Shift && !e.Alt)
                {
                    actionKey = CheckNormalKeys(e);
                }
                //
                //  Handle keyboard mapping for control and alt
                //
                if (!actionKey && e.Control && e.Alt && !e.Shift)
                {
                    actionKey = CheckAltControlKeys(e);
                }
                //
                //  Handle keyboard mapping for alt
                //
                if (!actionKey && !e.Control && e.Alt && !e.Shift)
                {
                    actionKey = CheckAltKeys(e);
                }
                //
                //  Check if we want to do something if the Ctrl key is down
                //
                if (!actionKey && e.Control && !e.Alt && !e.Shift)
                {
                    actionKey = CheckControlKeys(e);
                }
            }
            else
            {
                //
                //  Not online to the host so eat the keystroke and do nothing!
                //
                actionKey = true;
            }
            //
            //  Is this a keystroke that we are acting upon?
            //
            if (actionKey)
            {
                e.Handled = true;                   //indicate we handled this keystroke
                m_orginationKey = e;                //save the keystroke for task completion
                string s = "FunctionKeyPressed Event Finishing, Starting background worker";
                Debug.WriteLine(s);
                bgw_keyBoard.RunWorkerAsync();      //kick off the background task
            }
            else
            {
                ovApi.FunctionKeyPressedDone(e);        //re-enable OV's keyboard
            }
            Debug.WriteLine("Exiting FunctionKeyPressed Event");
        }

        /// <summary>
        /// Check normal keys without shift, alt or control modifiers to see
        /// if they are an action request that we need to act on
        /// </summary>
        /// <param name="e">Current keystroke from FunctionKeyPressedEvent</param>
        /// <returns>true if its an action or alert key</returns>
        private bool CheckNormalKeys(KeyEventArgs e)
        {
            bool actionKey = true;      //assume we take action on this function keystroke event
            switch (e.KeyCode)
            {
                default:
                    actionKey = false;          //nothing matched so allow emulation to process the key
                    break;
            }
            return actionKey;
        }
        /// <summary>
        /// Check keys that have the control modifier to see
        /// if they are an action request that we need to act on
        /// </summary>
        /// <param name="e">Current keystroke from FunctionKeyPressedEvent</param>
        /// <returns>true if its an action or alert key</returns>
        private bool CheckControlKeys(KeyEventArgs e)
        {
            bool actionKey = true;
            switch (e.KeyCode)
            {
                default:
                    actionKey = false;
                    break;
            }
            return actionKey;
        }
        /// <summary>
        /// Check keys that have the alt modifier to see
        /// if they are an action request that we need to act on
        /// </summary>
        /// <param name="e">Current keystroke from FunctionKeyPressedEvent</param>
        /// <returns>true if its an action or alert key</returns>
        private bool CheckAltKeys(KeyEventArgs e)
        {
            bool actionKey = false;
            switch (e.KeyCode)
            {
                //
                //  ALT-S, Invoke the ViewSys Scan Functionality
                //
                case Keys.S:
                    {
                        if (ovApi.CurrentScreenContext.Equals("viewsys"))
                        {
                            actionKey = true;
                        }
                        else
                        {
                            string caption = "Automated VIEWSYS Scan";
                            string msg = "You need to be in VIEWSYS context in order to perform this action";
                            DialogResult result = MessageBox.Show(msg, caption,
                                MessageBoxButtons.YesNo,
                                MessageBoxIcon.Warning,
                                MessageBoxDefaultButton.Button1,
                                MessageBoxOptions.DefaultDesktopOnly);
                        }
                    }
                    break;
                case Keys.D:
                    {
                        string s = "BeginInvoke for GuiMainFormVisible()";
                        Debug.WriteLine(s);
                        this.BeginInvoke(new MethodInvoker(delegate() { GuiMainFormVisible(); }));
                    }
                    break;
                default:
                    actionKey = false;
                    break;
            }
            return actionKey;
        }
        /// <summary>
        /// Check keys that have the alt and control modifiers to see
        /// if they are an action request that we need to act on
        /// </summary>
        /// <param name="e">Current keystroke from FunctionKeyPressedEvent</param>
        /// <returns>true if its an action or alert key</returns>
        private bool CheckAltControlKeys(KeyEventArgs e)
        {
            bool actionKey = true;
            switch (e.KeyCode)
            {
                default:
                    actionKey = false;
                    break;
            }
            return actionKey;
        }
        /// <summary>
        /// Diagnostic report for threading issues of the keystroke received
        /// </summary>
        /// <param name="e">Function key event arguments</param>
        /// <param name="EndAction">end action flag</param>
        private void DebugReportFunctionKey(KeyEventArgs e, bool EndAction)
        {
            string s = "";
            if (e.Alt)
                s += "Alt Key + ";
            if (e.Control)
                s += "Control Key + ";
            if (e.Shift)
                s += "Shift Key + ";
            s += e.KeyCode.ToString();
            if (e.Handled)
                s += " (handled) ";
            if (!EndAction)
                s += " FunctionKeyPressed Event: ";
            else
                s += " FunctionKeyPressedDone Notification: ";
            Debug.WriteLine(s);
        }


        /// <summary>
        /// Make the main form visible or invisible to the user.
        /// </summary>
        private void GuiMainFormVisible()
        {
            if (!this.ShowInTaskbar)
            {
                this.WindowState = FormWindowState.Normal;
                this.ShowInTaskbar = true;
            }
            else
            {
                this.WindowState = FormWindowState.Minimized;
                this.ShowInTaskbar = false;
            }
        }
        /// <summary>
        /// Size changed event handler, check if we just got minimized, if so hide our selfs from the taskbar!
        /// </summary>
        /// <param name="sender">this form object</param>
        /// <param name="e">default empty arguments</param>
        void MainForm_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.ShowInTaskbar = false;
            }
        }

        #endregion

        #region Background worker event handlers

        private KeyEventArgs m_orginationKey;
        /// <summary>
        /// Handles the RunWorkerCompleted event of the bgw_keyBoard component.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs"/> instance containing the event data.</param>
        private void bgw_keyBoard_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ovApi.EmitFunctionKey(TN6530FunctionKey.SHIFT_F1);
            ovApi.FunctionKeyPressedDone(m_orginationKey);
        }
        /// <summary>
        /// Handles the DoWork event of the bgw_keyBoard component.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs"/> instance containing the event data.</param>
        private void bgw_keyBoard_DoWork(object sender, DoWorkEventArgs e)
        {
            TN6530FunctionKey actionKey = TN6530FunctionKey.SHIFT_F1;
            do
            {
                ovApi.EmitFunctionKey(actionKey);
                Thread.Sleep(4000);
                int i = ((int)actionKey) + 1;
                actionKey = (TN6530FunctionKey)i;
            } while (!(actionKey == TN6530FunctionKey.SHIFT_F16));
        }

        #endregion

    }
}
